#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#       metricfilter.py
#
#       Copyright 2013 Jelle Smet development@smetj.net
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 3 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.
#
#

from wishbone import Actor

class MetricFilter(Actor):
    '''**MetricFilter is a MetricFactory filter to filter or rename metric names.***

    MetricFilter allows certain metrics to be dropped or metrics names to be
    rewritten in transit.  This can be practical when dealing with metrics coming
    from different sources.

    metric = {  "type":string,
                "time":time(),
                "source":string,
                "name":string,
                "value":string,
                "units":string,
                "tags":list
                }


    Parameters:

        - name (str):   The instance name when initiated.
        - meta (bool):  When True, drops data with version 128

    Queues:

        - inbox:    Incoming events.
        - outbox:   Outgoing events.
    '''

    def __init__(self, name):
        Actor.__init__(self, name)
        self.name=name

    def consume(self,doc):
        pass


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#       rsyslog.py
#
#       Copyright 2013 Jelle Smet development@smetj.net
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 3 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.
#
#

from wishbone import Actor
from wishbone.errors import QueueLocked
from gevent import monkey;monkey.patch_time()
from time import time
import json
import re

class Rsyslog(Actor):
    '''**Decode Rsyslog metrics.**

    This module takes JSON formatted metrics coming from Rsyslog and
    converts them to the metricfactory default format.

    Rsyslog seems to change frequently.  This module is expecting
    the metrics coming from Rsyslog 7.6.0

    The name of the metric will be point delimited.

    Parameters:

        - name (str):           The instance name.

        - source(str):          Allows to set the source manually.
                                Default: rsyslog


    Queues:

        - inbox:    Incoming events.

        - outbox:   Outgoing events.

    The template to use in Rsyslog is:

    template (name="metrics" type="string" string="%TIMESTAMP:::date-unixtimestamp% %fromhost% %msg%\n")
    '''

    # 1394101824 {"name":"imuxsock","submitted":2,"ratelimit.discarded":0,"ratelimit.numratelimiters":2}
    # 1394101824 {"name":"stats","processed":8988,"failed":0,"suspended":0,"suspended.duration":0,"resumed":0}
    # 1394101824 {"name":"local_logs","processed":3,"failed":0,"suspended":0,"suspended.duration":0,"resumed":0}
    # 1394101824 {"name":"logstash_logs","processed":3,"failed":0,"suspended":0,"suspended.duration":0,"resumed":0}
    # 1394101824 {"name":"imudp(*:514)","submitted":0}
    # 1394101824 {"name":"imudp(*:514)","submitted":0}
    # 1394101824 {"name":"imtcp(514)","submitted":0}
    # 1394101824 {"name":"resource-usage","utime":153976,"stime":280957,"maxrss":4068,"minflt":437,"majflt":0,"inblock":0,"oublock":24,"nvcsw":1591,"nivcsw":15}
    # 1394101824 {"name":"logstash_logs[DA]","size":0,"enqueued":0,"full":0,"discarded.full":0,"discarded.nf":0,"maxqsize":0}
    # 1394101824 {"name":"logstash_logs","size":0,"enqueued":3,"full":0,"discarded.full":0,"discarded.nf":0,"maxqsize":1}
    # 1394101824 {"name":"main Q","size":10,"enqueued":9001,"full":0,"discarded.full":0,"discarded.nf":0,"maxqsize":12}
    # 1394101824 {"name":"imudp(w0)","called.recvmmsg":0,"called.recvmsg":0,"msgs.received":0}


    def __init__(self, name, source="rsyslog"):

        Actor.__init__(self, name, setupbasic=True)
        self.logging.info("Initialized")
        self.source=source
        self.prev_metric={}

    def consume(self, event):
        try:
            for metric in self.__extractMetrics(event["data"].rstrip()):
                self.queuepool.outbox.put({"header":event["header"], "data":metric})
        except QueueLocked:
            self.queuepool.inbox.rescue(event)
            self.queuepool.outbox.waitUntillPutAllowed()

    def __extractMetrics(self, data):

        try:
            (time, hostname, j) = re.match('(^\d*) (.*?) (.*)', data).groups()
            j = json.loads(j)
            for (name, value) in self.__extractIndividualMetric(hostname, j):
                yield self.__formatMetric(time, name, value)
                rate = self.__getRate(time, name, value)
                yield self.__formatMetric(time, name.replace('.total','.rate'), rate)
        except Exception as err:
            self.logging.warning("Bad Rsyslog metric format.  Check the Rsyslog template to use.  Reason: %s"%(err))

    def __extractIndividualMetric(self, hostname, data):

        keys = data.keys()
        keys.remove("name")
        name = self.__scrubMetricName(data["name"])
        for item in keys:
            yield "%s.%s.%s.total"%(hostname, name, item.replace('.','')), data[item]

    def __scrubMetricName(self, name):
        for character in [")", "*", ":", "]", " "]:
            name = name.replace(character,"")
        for character in ["(","["]:
            name = name.replace(character,"_")
        return name.lower()

    def __getRate(self, time, name, value):
        try:
            previous_value = self.prev_metric[name]["value"]
            previous_time  = self.prev_metric[name]["time"]
        except KeyError:
            self.prev_metric[name]={"value":value,"time":time}
            return 0
        else:
            self.prev_metric[name]={"value":value,"time":time}
            try:
                return (int(value) - int(previous_value))/(int(time) - int(previous_time))
            except ZeroDivisionError:
                return 0

    def __formatMetric(self, timestamp, name, value):

        return ((timestamp, "rsyslog", self.source, name, value, '', ()))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#       rabbitmq.py
#
#       Copyright 2013 Jelle Smet development@smetj.net
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 3 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.
#
#

from wishbone import Actor
from wishbone.errors import QueueLocked
from gevent import monkey;monkey.patch_time()
from time import time
import json

class RabbitMQ(Actor):
    '''**Decode RabbitMQ metrics.**

    This module takes a JSON formatted coming from a RabbitMQ
    clusternode and converts the metrics into the common format.

    The name of the metric will be point delimited.

    Parameters:

        - name (str):           The instance name.

        - source(str):          Allows to set the source manually.
                                Default: rabbitmq

    Queues:

        - inbox:    Incoming events.
        - outbox:   Outgoing events.

    It's possible to process metrics coming from following resources:

        - /api/nodes
        - /api/queues
    '''

    def __init__(self, name, source="rabbitmq"):
        Actor.__init__(self, name, setupbasic=True)
        self.logging.info("Initialized")
        self.source=source

    def consume(self, event):

        try:
            data=json.loads(event["data"])
            for metric in self.extractMetrics(data):
                self.queuepool.outbox.put({"header":event["header"], "data":metric})
        except ValueError as err:
            self.logging.error("Problem reading JSON data.  Reason: %s"%(err))
        except QueueLocked:
            self.queuepool.inbox.rescue(event)
            self.queuepool.outbox.waitUntillPutAllowed()
        except Exception as err:
            self.logging.error("Unexpected error encountered possibly due to the event format. Event dropped.  Reason: %s"%(err))

    def extractMetrics(self, data):
        #(time, type, source, name, value, unit, (tag1, tag2))
        #(1381002603.726132, 'wishbone', 'hostname', 'queue.outbox.in_rate', 0, '', ())
        if len(data) == 0:
            return

        timestamp=time()
        if "os_pid" in data[0]:
            #Got metrics from /api/nodes
            for node in data:
                for metric in [ "fd_used","fd_total","sockets_used","sockets_total","mem_used","mem_limit","disk_free_limit","disk_free","proc_used","proc_total","uptime","run_queue","processors" ]:
                    yield (timestamp,
                        "rabbitmq",
                        self.source,
                        "node.%s.%s"%(node["name"].split("@")[1], metric),
                        node.get(metric,0),
                        '',
                        ())

        elif "backing_queue_status" in data[0]:
            #Got metrics from /api/queues
            for queue in data:
                for metric in self.__crawlDictionary(timestamp, queue, "%s.queue.%s"%(queue["vhost"],queue["name"])):
                        yield metric

        elif data[1]["name"] == "amq.direct":
            #got metrics from /api/exchanges
            for exchange in data:
                if exchange["name"] == "":
                    exchange["name"] = "_AMQP_default_"
                for metric in self.__crawlDictionary(timestamp, exchange, "%s.exchange.%s"%(exchange["vhost"],exchange["name"])):
                    yield metric


    def __formatMetric(self, timestamp, name, value):
        return (timestamp, "elasticsearch", self.source, name, value, '', ())

    def __crawlDictionary(self, timestamp, dictionary,  breadcrumbs=""):

        for k, v in dictionary.iteritems():
            b = "%s.%s"%(breadcrumbs, k)
            if isinstance(v, dict):
                 for metric in self.__crawlDictionary(timestamp, v, b):
                    yield metric
            elif isinstance(v, list):
                continue
            elif isinstance(v, (int, long, float, complex)):
                yield self.__formatMetric(timestamp, b, v)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#       graphite.py
#
#       Copyright 2013 Jelle Smet development@smetj.net
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 3 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.
#
#

from wishbone import Actor
from time import time
from random import randint, choice
import string
from gevent import sleep
from gevent.socket import gethostname
from gevent import spawn
from gevent.event import Event

class Hammer(Actor):
    '''**Generates random metric events.**

    Generates random metrics events in internal format with the purpose of
    testing.  Metrics are generated in batches.  One batch is a list of
    dictionaries.

    (time, type, source, name, value, unit, (tag1, tag2))

    Parameters:

        - name (str):       The instance name when initiated.

        - batch (int):      The number of batches to produce.  0 is unlimited.
                            (default: 0)

        - batch_size(int):  The number of unique data sets in 1 batch.
                            (default: 1)

        - set_size(int):    The number of unique metrics per set.
                            (default: 1)

        - sleep (float):    The time to sleep in between generating batches.
                            (default: 1)

        - value (int):      The maximum of the randomized metric value (default 1).

        - prefix (str):     The top level name.
                            (default: hammer)

    Queues:

        - outbox:    Generated metrics.


    Metrics names will have following name structure:

        <prefix>.set_<batch_size>.metric_<set_size>


        hammer.set_0.metric_0 34534 1382173076

    '''

    def __init__(self, name, batch=0, batch_size=1, set_size=1, sleep=1, value=1, prefix='hammer'):
        Actor.__init__(self, name, setupbasic=False)
        self.name=name

        self.batch=batch
        self.batch_size=batch_size
        self.set_size=set_size
        self.sleep_value=float(sleep)
        self.value=value
        self.prefix=prefix

        if batch == 0:
            self.generateNextBatchAllowed = self.__returnTrue
        else:
            self.generateNextBatchAllowed = self.__evaluateCounter

        if sleep == 0:
            self.doSleep = self.__doNoSleep
        else:
            self.doSleep = self.__doSleep

        self.pause = Event()
        self.pause.set()

    def preHook(self):
        sleep(1)
        spawn(self.generate)

    def generate(self):
        switcher = self.getContextSwitcher(100)
        batch_counter=0
        while switcher():
            if self.generateNextBatchAllowed(batch_counter) == True:
                for event in self.generateBatch():
                    try:
                        self.queuepool.outbox.put({"header":{}, "data":event})
                    except:
                        self.queuepool.outbox.waitUntilPutAllowed()
                        self.queuepool.outbox.put({"header":{}, "data":event})
                batch_counter+=1
            else:
                break
            self.doSleep()
        self.logging.warn('Reached the batch_size of %s.  Not generating any further metrics.'%(self.batch_size))

    def generateBatch(self):
        #(time, type, source, name, value, unit, (tag1, tag2))
        #(1381002603.726132, 'wishbone', 'wishbone', 'queue.outbox.in_rate', 0, '', ())
        timestamp = time()

        for set_name in xrange(self.batch_size):
            for metric_name in xrange(self.set_size):
                self.pause.wait()
                yield (timestamp, 'test', 'hammer', '%s.set_%s.metric_%s'%(self.prefix, set_name, metric_name), randint(0, self.value), '', () )

    def __evaluateCounter(self, counter):
        if counter == self.batch:
            return False
        else:
            return True

    def __returnTrue(self, counter):
        return True

    def __doSleep(self):
        sleep(self.sleep_value)

    def __doNoSleep(self):
        sleep(0.0001)

    def enableThrottling(self):
        self.pause.clear()

    def disableThrottling(self):
        self.pause.set()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#       skeleton.py
#
#       Copyright 2013 Jelle Smet development@smetj.net
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 3 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.
#
#

from wishbone import Actor
from wishbone.errors import QueueLocked
from gevent import monkey;monkey.patch_time()
from time import time
import json

class Elasticsearch(Actor):
    '''**Decode Elasticsearch metrics.**

    This module takes a JSON formatted coming from an Elasticsearch
    clusternode and converts the metrics into the common format.

    The name of the metric will be point delimited.

    Parameters:

        - name (str):           The instance name.

        - source(str):          Allows to set the source manually.
                                Default: elasticsearch

        - indices(list):        The indices to include when polling
                                /_stats
                                Default: []

    Queues:

        - inbox:    Incoming events.
        - outbox:   Outgoing events.

    This module deals with JSON documents coming from:

        - /_stats
        - /_cluster/stats
        - /_nodes/stats

    Feeding this module any other stats data will not work.
    Requires Elasticsearch >= 1.0.0
    '''

    def __init__(self, name, source="elasticsearch", indices=[]):
        Actor.__init__(self, name, setupbasic=True)
        self.logging.info("Initialized")
        self.source=source
        self.indices=indices

    def consume(self, event):

        try:
            data=json.loads(event["data"])
            for metric in self.extractMetrics(data):
                self.queuepool.outbox.put({"header":event["header"], "data":metric})
        except ValueError as err:
            self.logging.error("Problem reading JSON data.  Reason: %s"%(err))
        except QueueLocked:
            self.queuepool.inbox.rescue(event)
            self.queuepool.outbox.waitUntillPutAllowed()
        except Exception as err:
            self.logging.error("Unexpected error encountered possibly due to the event format. Event dropped.  Reason: %s"%(err))

    def __formatMetric(self, timestamp, name, value):
        return (timestamp, "elasticsearch", self.source, name, value, '', ())

    def __crawlDictionary(self, timestamp, dictionary,  breadcrumbs=""):

        for k, v in dictionary.iteritems():
            b = "%s.%s"%(breadcrumbs, k)
            if isinstance(v, dict):
                 for metric in self.__crawlDictionary(timestamp, v, b):
                    yield metric
            elif isinstance(v, list):
                continue
            elif isinstance(v, (int, long, float, complex)):
                yield self.__formatMetric(timestamp, b, v)

    def extractMetrics(self, data):
        #(time, type, source, name, value, unit, (tag1, tag2))
        #(1381002603.726132, 'wishbone', 'hostname', 'queue.outbox.in_rate', 0, '', ())

        timestamp=time()
        if all (False for k in [ "_shards", "_all", "indices" ] if not k in data):
            #We have received metrics from /_stats
            for element in ["_shards","_all"]:
                for metric in self.__crawlDictionary(timestamp, data[element], "indices.%s"%(element)):
                    yield metric
            for index in self.indices:
                try:
                    for metric in self.__crawlDictionary(timestamp, data["indices"][index], index):
                        yield metric
                except KeyError:
                    self.logging.error("Index %s does not exist."%(metric))
        elif all (False for k in [ "timestamp", "cluster_name", "status", "indices", "nodes" ] if not k in data):
            #We have received metrics from /_cluster/stats
            for element in ["indices","nodes"]:
                for metric in self.__crawlDictionary(data["timestamp"]/1000, data[element], "cluster.%s"%(element)):
                    yield metric

        elif all (False for k in [ "cluster_name", "nodes" ] if not k in data):
            #We have received metrics from /_nodes/stats
            for node in data["nodes"]:
                for metric in self.__crawlDictionary(data["nodes"][node]["timestamp"]/1000, data["nodes"][node], "nodes.%s"%(data["nodes"][node]["name"])):
                    yield metric

        elif all (False for k in [ "cluster_name", "status", "timed_out" ] if not k in data):
            #We have received metrics from /_cluster/health
            for metric in self.__crawlDictionary(timestamp, data, "cluster.health"):
                yield metric
        else:
            self.logging.error("Unrecognized dataset.  Are you sure you have polled the right resource?")


#!/usr/bin/env python
#
# -*- coding: utf-8 -*-
#
#  main.py
#
#  Copyright 2013 Jelle Smet <development@smetj.net>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#
#

from wishbone.bootstrap import BootStrap
import sys

def main():
    try:
        BootStrap()
    except Exception as err:
        sys.stderr.write("Failed to bootstrap instance.  Reason: %s\n"%(err))
        sys.stderr.flush()
        sys.exit(1)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#       decodeganglia.py
#
#       Copyright 2013 Jelle Smet development@smetj.net
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 3 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.
#
#

#A big thanks to:
# Michael Rose https://bitbucket.org/michaelrose/ganglia-xdr-parser/src/da033e2bed3e/ganglia_xdr_parser.py
# Patrick Debois https://gist.github.com/1376525

from wishbone import Actor
import xdrlib
from time import time
from gevent.monkey import patch_time;patch_time()

class DecodeGangliaException(Exception):
    def __init__(self, value):
        self.value = value
    def __str__(self):
        return repr(self.value)

class Ganglia(Actor):
    '''**DecodeGanglia is MetricFactory module which decodes Ganglia metrics into
    a MetricFactory format.***

    Receives xdr formatted data coming from a Ganglia client and decodes it
    into Wishbone format.

    (1381002603.726132, 'hadoop', 'hostname', 'queue.outbox.in_rate', 0, '', ())

    Parameters:

        - name (str):   The instance name when initiated.
        - meta (bool):  When True, drops data with version 128

    Queues:

        - inbox:    Incoming events.
        - outbox:   Outgoing events.
    '''

    def __init__(self, name, meta=False):
        Actor.__init__(self, name)
        self.name=name
        self.meta=meta

    def parsePacket(self, data):
        unpacker = xdrlib.Unpacker(data)
        version = unpacker.unpack_int()

        if version in (132,133,134):
            return self.doHeartBeatPacket(unpacker,version)
        elif version == 128:
            if self.meta == True:
                return self.doMetaPacket(unpacker,version)
            else:
                raise DecodeGangliaException("Version 128 is ignored.")
        else:
            raise Exception("Unknown version number: %s"%version)

    def consume(self,event):
        try:
            data = self.parsePacket(event["data"])
            event['data']=(time(), "ganglia", data["hostname"], data["metric_name"], data["value"], "", ())
            self.queuepool.outbox.put(event)
        except Exception as err:
            self.logging.debug ( "Failed to decode package. Reason: %s" %err )
        except DecodeGangliaException as err:
            self.logging.debug ( err )

    def doMetaPacket(self, unpacker, version):
        data = {"version":version,
                "hostname":unpacker.unpack_string(),
                "metric_name":unpacker.unpack_string(),
                "spoofed_hostname":bool(unpacker.unpack_int()),
                "metric_type":unpacker.unpack_string(),
                "metric_name2":unpacker.unpack_string(),
                "units":unpacker.unpack_string(),
                "slope":unpacker.unpack_int(),
                "tmax":unpacker.unpack_int(),
                "dmax":unpacker.unpack_int(),
                "metrics":{}
                }
        remaining = unpacker.unpack_int()
        for i in range(remaining):
            data["metrics"][unpacker.unpack_string()]=unpacker.unpack_string()
        unpacker.done()
        if data != None:
            return data
        else:
            raise Exception ("Empty dataset")

    def doHeartBeatPacket(self, unpacker, version):
        data = {"version":version,
                "hostname":unpacker.unpack_string(),
                "metric_name":unpacker.unpack_string(),
                "spoof":bool(unpacker.unpack_int()),
                "format":unpacker.unpack_string()
                }

        if data["format"].endswith('f'):
            data["value"] = unpacker.unpack_float()
        elif data["format"].endswith('u') or data["format"].endswith('d'):
            data["value"] = unpacker.unpack_int()
        elif data["format"].endswith('s'):
            data["value"] = unpacker.unpack_string()
        unpacker.done()
        if data != None:
            return data
        else:
            raise Exception ("Empty dataset")




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#       decodemodgearman.py
#
#       Copyright 2013 Jelle Smet development@smetj.net
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 3 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.
#
#

from wishbone import Actor
import re
import sys

class ModGearman(Actor):
    '''**A module to decodes Mod_Gearman metrics into the default format.**

    Receives Nagios spool directory formatted data and decodes it into a
    MetricFactory format.

    metric = {  "type":string,
                "time":time(),
                "source":string,
                "name":string,
                "value":string,
                "units":string,
                "tags":list
                }


    Parameters:

        - name (str):   The instance name when initiated.

    Queues:

        - inbox:    Incoming events in Nagios spool directory format.
        - outbox:   Outgoing events in MetricFactory format.
    '''

    def __init__(self, name):
        Actor.__init__(self, name)
        self.regex=re.compile('(.*?)(\D+)$')

    def consume(self,event):
        try:
            for metric in self.decodeMetrics(event["data"]):
                self.queuepool.outbox.put({"header":event["header"],"data":metric})
        except Exception as err:
            self.logging.warn('Malformatted performance data received. Reason: %s'%(err))

    def decodeMetrics(self, data):

        #DATATYPE::SERVICEPERFDATA\t
        #TIMET::1383478571\t
        #HOSTNAME::dev-umi-master-101.flatns.net\t
        #SERVICEDESC::Default Processes\t
        #SERVICEPERFDATA::check_multi::check_multi::plugins=5 time=0.058422\t
        #SERVICECHECKCOMMAND::check:nagios.multicheck.status\t
        #SERVICESTATE::0\t
        #SERVICESTATETYPE::1\n\n\n\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00

        metadata={}
        for element in data.split("\t"):
            parts = re.search('(.*?)::(.*)',element)
            metadata[parts.group(1).lower()]=parts.group(2)

        if 'serviceperfdata' in metadata:
            metric_type = "serviceperfdata"
        if 'hostperfdata' in metadata:
            metric_type = "hostperfdata"

        for metric in re.sub('.*::', '', metadata[metric_type]).split(' '):
            (key,value)=metric.split('=')
            value=value.split(";")[0]
            value_unit = self.regex.search(value)

            if value_unit == None:
                units=''
            elif value_unit.group(2):
                value=value_unit.group(1)
                units=value_unit.group(2)
            else:
                units=''
            tags=[]
            tags.append(self.__filter(metadata.get("servicecheckcommand",metadata.get("hostcheckcommand",""))))
            if "servicedesc" in metadata:
                tags.append(self.__filter(metadata["servicedesc"]))
                tags.append("servicecheck")
                basename=self.__filter(metadata["servicedesc"])
            else:
                basename="hostcheck"
                tags.append("hostcheck")

            #(1381002603.726132, 'wishbone', 'hostname', 'queue.outbox.in_rate', 0, '', ())
            yield((metadata["timet"], "nagios", self.__filter(metadata["hostname"]), "%s.%s"%(basename, self.__filter(key)), value, units, tuple(tags)))

    def __filter(self, name):
        '''Filter out problematic characters.

        This should become a separate module allowing the user to define filter rules
        from a bootstrap file and most likely become a separate module.
        '''

        name=name.replace("'",'')
        name=name.replace('"','')
        name=name.replace('!(null)','')
        name=name.replace(" ","_")
        name=name.replace("/","_")
        return name.lower()